ColorPicker = class(Turbine.UI.Lotro.Window);

function ColorPicker:Constructor(color, dimension)
    Turbine.UI.Lotro.Window.Constructor(self);

    if (color) then
        color = Color(color.R, color.G, color.B);
    else
        color = Color(0.5, 1.0, 1.0);
    end
    if (not dimension) then
        dimension = "V";
    end
    
    self:SetSize(407, 316);
    self:SetVisible(true);
    L:SetContext("/ColorPicker");
    self:SetText(L:GetText("DefaultTitle"));
    self:SetResizable(true);

    self.color = Color(1, 0.5, 0.5, 0.5);
    self.originalColor = Color(color.R, color.G, color.B);
    
    self.palette = Palette(self, "G", "B", self.color);
    self.palette.ColorChanged = function()
        self:PaletteChanged();
    end

    self.slider = Slider(self, "R", self.color);
    self.slider.ColorChanged = function()
        self:SliderChanged();
    end

    local radioButtonWidth, radioButtonHeight = 127, 20;
    self.radioContainer = Turbine.UI.Window();
    self.radioContainer:SetWidth(radioButtonWidth);
    self.radioContainer:SetVisible(true);
    self.radioButtons = {};
    
    self.radioButtons["R"] = RadioButton(self.radioContainer, L:GetText("Red"), true);
    self.radioButtons["R"]:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.radioButtons["R"]:SetSize(radioButtonWidth, radioButtonHeight);
    self.radioButtons["R"].Clicked = function()
        self:SelectDimension("R");
    end
    local top = radioButtonHeight;

    self.radioButtons["G"] = RadioButton(self.radioContainer, L:GetText("Green"), false);
    self.radioButtons["G"]:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.radioButtons["G"]:SetTop(top);
    self.radioButtons["G"]:SetSize(radioButtonWidth, radioButtonHeight);
    self.radioButtons["G"].Clicked = function()
        self:SelectDimension("G");
    end
    top = top + radioButtonHeight;

    self.radioButtons["B"] = RadioButton(self.radioContainer, L:GetText("Blue"), false);
    self.radioButtons["B"]:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.radioButtons["B"]:SetTop(top);
    self.radioButtons["B"]:SetSize(radioButtonWidth, radioButtonHeight);
    self.radioButtons["B"].Clicked = function()
        self:SelectDimension("B");
    end
    top = top + 30;

    self.radioButtons["H"] = RadioButton(self.radioContainer, L:GetText("Hue"), false);
    self.radioButtons["H"]:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.radioButtons["H"]:SetTop(top);
    self.radioButtons["H"]:SetSize(radioButtonWidth, radioButtonHeight);
    self.radioButtons["H"].Clicked = function()
        self:SelectDimension("H");
    end
    top = top + radioButtonHeight;

    self.radioButtons["S"] = RadioButton(self.radioContainer, L:GetText("Saturation"), false);
    self.radioButtons["S"]:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.radioButtons["S"]:SetTop(top);
    self.radioButtons["S"]:SetSize(radioButtonWidth, radioButtonHeight);
    self.radioButtons["S"].Clicked = function()
        self:SelectDimension("S");
    end
    top = top + radioButtonHeight;

    self.radioButtons["V"] = RadioButton(self.radioContainer, L:GetText("Value"), false);
    self.radioButtons["V"]:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.radioButtons["V"]:SetTop(top);
    self.radioButtons["V"]:SetSize(radioButtonWidth, radioButtonHeight);
    self.radioButtons["V"].Clicked = function()
        self:SelectDimension("V");
    end
    top = top + 34;
    
    RadioButton.LinkPeers({self.radioButtons["R"], self.radioButtons["G"], self.radioButtons["B"], self.radioButtons["H"], self.radioButtons["S"], self.radioButtons["V"]});
    self.radioContainer:SetZOrder(self:GetZOrder() + 5);
    self.radioContainer:SetHeight(top);
    
    self.hex = Turbine.UI.Lotro.TextBox();
    self.hex:SetParent(self);
    self.hex:SetSize(math.floor(radioButtonWidth * 2 / 3 + 0.5), radioButtonHeight);
    self.hex:SetFont(Turbine.UI.Lotro.Font.Verdana14);
    self.hex:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    self.hex:SetWantsUpdates(true);
    self.hex.Update = function()
        local hexText = self.hex:GetText();
        if (self.hex.prevText ~= hexText) then
            self.hex.prevText = hexText;
            self:HexChanged();
        end
    end
    
    self.okButton = Turbine.UI.Lotro.Button();
    self.okButton:SetParent(self);
    self.okButton:SetSize(math.floor(radioButtonWidth / 2 + 0.5), radioButtonHeight);
    self.okButton:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
    self.okButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    self.okButton:SetText(L:GetText("Ok"));
    self.okButton.Click = function()
        self.okButtonClicked = true;
        if (self.Accepted) then
            self:Accepted();
        end
        self:Close();
    end
    
    self.changingSize = false;
    self:DoLayout();
    self:SetColor(color);
    self.radioButtons[dimension]:MouseClick();
end

function ColorPicker:DoLayout()
    local titleHeight = 14;
    local marginSize = 23;
    local spacing = 10;
    local sliderWidth = 20;
    
    local top = titleHeight + marginSize;
    local left = marginSize;
    local radioContainerWidth, radioContainerHeight = self.radioContainer:GetSize();
    local minPaletteSize = radioContainerHeight + 50;
    local paletteSize = self:GetWidth() - (marginSize * 2) - (spacing * 2) - self.radioContainer:GetWidth() - sliderWidth;
    if (paletteSize < minPaletteSize) then
        self:SetWidth(self:GetWidth() + (minPaletteSize - paletteSize));
        paletteSize = minPaletteSize;
    end
    self.palette:SetPosition(left, top);
    self.palette:SetSize(paletteSize, paletteSize);
    self.palette:SetStretchMode(1);

    left = left + paletteSize + spacing;
    self.slider:SetPosition(left, top);
    self.slider:SetSize(sliderWidth, paletteSize);
    self.slider:SetStretchMode(1);

    left = left + sliderWidth + spacing;
    top = top + math.floor(((paletteSize - 50 - radioContainerHeight) / 2) + 0.5);
    local l, t = self:PointToScreen(left, top);
    self.radioContainer:SetZOrder(self:GetZOrder() + 1);
    self.radioContainer:SetPosition(l, t);
    
    top = titleHeight + marginSize + paletteSize - self.okButton:GetHeight();
    self.okButton:SetPosition(left + math.floor(radioContainerWidth / 4 + 0.5), top);
    
    top = top - 8 - self.hex:GetHeight();
    self.hex:SetPosition(left + math.floor(radioContainerWidth / 6 + 0.5), top);

    self:SetHeight(titleHeight + marginSize + paletteSize + marginSize);
end

function ColorPicker:SelectDimension(sliderDimension)
    local paletteXDimension, paletteYDimension = Color.GetOtherDimensions(sliderDimension);
    self.slider:SetDimension(sliderDimension);
    self.palette:SetDimensions(paletteXDimension, paletteYDimension);
    
    -- Update display
    self:SetColor(self.color);
end

function ColorPicker:HexChanged()
    local R, G, B = string.match(self.hex:GetText(), "^(%x%x)(%x%x)(%x%x)$");
    if (R and G and B) then
        self.hex:SetBackColor(Color(1, 1, 1, 1));
        if (not self.updatingHex) then
            local newColor = Color(1, 0, 0, 0);
            newColor.R = tonumber('0x' .. R) / 255;
            newColor.G = tonumber('0x' .. G) / 255;
            newColor.B = tonumber('0x' .. B) / 255;
            self:SetColor(newColor);
            if (self.ColorChanged) then
                self:ColorChanged();
            end
        end
    else
        self.hex:SetBackColor(Color(1, 1, 0, 0));
    end
end

function ColorPicker:SetColor(color)
    self.slider:SetColor(color);
    self.palette:SetColor(color);
    self:PaletteChanged();
    self:SliderChanged();
end

function ColorPicker:GetColor()
    return self.color;
end

function ColorPicker:UpdateHex()
    local R, G, B, H, S, V;
    R = math.floor(self.color.R * 255 + 0.5)
    G = math.floor(self.color.G * 255 + 0.5)
    B = math.floor(self.color.B * 255 + 0.5)
    self.updatingHex = true;
    self.hex:SetText(string.format("%2.2X%2.2X%2.2X", R, G, B));
    self.hex.prevText = self.hex:GetText();
    self.updatingHex = false;
    R = math.floor(self.color.R * 100 + 0.5);
    G = math.floor(self.color.G * 100 + 0.5);
    B = math.floor(self.color.B * 100 + 0.5);
    H, S, V = self.color:GetHSV();
    H = math.floor(H * 360 + 0.5);
    S = math.floor(S * 100 + 0.5);
    V = math.floor(V * 100 + 0.5);
    L:SetContext("/ColorPicker");
    self.radioButtons["R"]:SetText(L:GetText("Red") .. ": " .. tostring(R) .. "%");
    self.radioButtons["G"]:SetText(L:GetText("Green") .. ": " .. tostring(G) .. "%");
    self.radioButtons["B"]:SetText(L:GetText("Blue") .. ": " .. tostring(B) .. "%");
    self.radioButtons["H"]:SetText(L:GetText("Hue") .. ": " .. tostring(H) .. "°");
    self.radioButtons["S"]:SetText(L:GetText("Saturation") .. ": " .. tostring(S) .. "%");
    self.radioButtons["V"]:SetText(L:GetText("Value") .. ": " .. tostring(V) .. "%");
end

function ColorPicker:SliderChanged()
    self.slider:GetColor(self.color);
    self:UpdateHex();
    self.palette:SetGamut(self.color);
    self.palette:SetSpotColor(self.color);
    if (self.ColorChanged) then
        self:ColorChanged();
    end
end

function ColorPicker:PaletteChanged()
    self.palette:GetColor(self.color);
    self:UpdateHex();
    self.palette:SetSpotColor(self.color);
    self.slider:SetGamut(self.color);
    if (self.ColorChanged) then
        self:ColorChanged();
    end
end

function ColorPicker:SizeChanged()
    if (not self.changingSize) then
        self.changingSize = true;
        self:DoLayout();
        self.changingSize = false;
    end
end

function ColorPicker:PositionChanged()
    self:DoLayout();
end

function ColorPicker:Closing()
    if (not self.okButtonClicked) then
        -- User closed the window.  Revert to original color.
        self.color = self.originalColor;
        self:ColorChanged();
        if (self.Canceled) then
            self:Canceled();
        end
    end
    self.palette:Close();
    self.slider:Close();
    self.radioContainer:Close();
end